@@ -4,7 +4,7 @@ import threading
 import time
 from enum import Enum, auto
 from pathlib import Path
-from typing import List, Optional, Dict
+from typing import List, Optional, Dict, Any
 
 from apscheduler.events import EVENT_JOB_EXECUTED, EVENT_JOB_ERROR, \
     EVENT_JOB_ADDED, EVENT_JOB_SUBMITTED, EVENT_JOB_REMOVED
@@ -19,6 +19,7 @@ from mysql.connector import Error as mysqlError
 from pydantic import BaseModel
 from telegram import Bot
 from datetime import datetime, timedelta
+import tzlocal
 
 from .mysql_basic import *
 
@@ -64,6 +65,8 @@ class JobInfo(BaseModel):
     module: str  # name of the module, where to find the callback function
     args: Optional[List] = []  # args to pass to the function
     kwargs: Optional[Dict] = {}  # kwargs to pass to the function
+    pydantic_args: Optional[List[tuple[str, dict]]] = None  # args tuple of [] will be first parsed by model.parse_obj
+    pydantic_kwargs: Optional[Dict[str, tuple[str, Dict]]] = None  # kwargs to pass to the function
     trigger: Optional[AllowedTrigger] = AllowedTrigger.date  # the trigger object that controls the schedule of this job
     trigger_config: dict = dict()  # configuration of the triggers
     coalesce: bool = False  # whether to only run the job once when several run times are due
@@ -182,6 +185,17 @@ class Task(BaseModel):
                 msg += f"\n\nduration:\n<code>{duration}</code>"
                 Task.send_bot_info(self, msg)
 
+    def execution_info_end_with_error(self, error: BaseException, duration: timedelta = None):
+        query = f"INSERT INTO db_global.Executions(task_id, result_msg, duration) VALUES (%s,%s,%s)"
+        error_msg = f"{type(error).__name__}: {error}"
+        _ = execute_query(query, (self.id, error_msg, str(duration)))
+        if self.job_info.bot_info is not None:
+            if self.status.value >= self.job_info.bot_info.info_level.value:
+                msg = f"❌ execution finished with error\n"
+                msg += f"<code>{self.__repr__()}</code>"
+                msg += f"\n\nerror:\n<code>{error_msg}</code>"
+                Task.send_bot_info(self, msg)
+
     def update(self, **kwargs):
         if "status" in kwargs and kwargs["status"] != self.status:
             self.status = kwargs["status"]
@@ -210,7 +224,7 @@ class Task(BaseModel):
 
 
 class ScheduleThread(threading.Thread):
-    def __init__(self, target_system: str, sleep_between_checks_sec=0.3):
+    def __init__(self, target_system: str, time_zone: str = None, sleep_between_checks_sec=0.3):
         super().__init__()
         self.mysql_cnx = create_connection()
         self.cease_continuous_run = threading.Event()
@@ -225,8 +239,13 @@ class ScheduleThread(threading.Thread):
         self.scheduler.add_listener(self.listen_execution_start, EVENT_JOB_SUBMITTED)
         self.scheduler.add_listener(self.listen_execution_ends, EVENT_JOB_EXECUTED | EVENT_JOB_ERROR)
         self.scheduler.add_listener(self.listen_job_removed, EVENT_JOB_REMOVED)
+        if time_zone:
+            self.time_zone = time_zone
+        else:
+            self.time_zone = str(tzlocal.get_localzone())
         self.tasks: List[Task] = list()
 
+
     def _add_task(self, task: Task):
         self.tasks.append(task)
 
@@ -260,7 +279,7 @@ class ScheduleThread(threading.Thread):
 
         # first send info, then change task status to keep the execution state
         if event.exception:
-            task.execution_info_end(event.exception, None)
+            task.execution_info_end_with_error(event.exception, None)
         else:
             task.execution_info_end(event.retval[0], event.retval[1])
 
@@ -324,9 +343,9 @@ class ScheduleThread(threading.Thread):
 
     def add_schedule_job(self, task: Task) -> Job:
         if task.job_info.trigger == AllowedTrigger.date:
-            trigger = DateTrigger(**task.job_info.trigger_config)
+            trigger = DateTrigger(**task.job_info.trigger_config, timezone=self.time_zone)
         elif task.job_info.trigger == AllowedTrigger.interval:
-            trigger = IntervalTrigger(**task.job_info.trigger_config)
+            trigger = IntervalTrigger(**task.job_info.trigger_config, timezone=self.time_zone)
         else:
             raise RuntimeError("Trigger not found!")
 
@@ -334,7 +353,10 @@ class ScheduleThread(threading.Thread):
             executor = 'background'
         else:
             executor = 'default'
-            # max. instances set to 1 per default, so nothing to do here?
+            # ignore user setting and set max instances to 1 per default
+            if task.job_info.max_instances != 1:
+                logger.warning("setting max instances to 1 since this is not a background job")
+                task.job_info.max_instances = 1
         # send bot info
         if task.job_info.bot_info is not None:
             msg = f"{task.status.symbol} Task created!\n"
@@ -354,6 +376,12 @@ class ScheduleThread(threading.Thread):
         # update the task
         tic = datetime.now()
         task = Task.get_task_from_id(task.id)
+        if task.job_info.pydantic_args is not None:
+            for pyd_arg in task.job_info.pydantic_args:
+                task.job_info.args += getattr(sys.modules[task.job_info.module], pyd_arg[0]).parse_obj(pyd_arg[1])
+        if task.job_info.pydantic_kwargs is not None:
+            for key, pyd_kwarg in task.job_info.pydantic_kwargs.items():
+                task.job_info.kwargs[key] = getattr(sys.modules[task.job_info.module], pyd_kwarg[0]).parse_obj(pyd_kwarg[1])
         result = getattr(sys.modules[task.job_info.module], task.job_info.callback)(*task.job_info.args,
                                                                                     **task.job_info.kwargs)
         duration = datetime.now() - tic
